package cupidonline.persistence;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import cupidonline.model.Colore;
import cupidonline.model.Persona;
import cupidonline.model.Sesso;
import cupidonline.model.SegnoZodiacale;

public class MyIscrittiReader implements IscrittiReader
{
	@Override
	public Map<String,Persona> caricaIscritti(Reader reader) throws IOException, BadFileFormatException
	{
		Map<String,Persona> mappaIscritti = new HashMap<>();
		BufferedReader fin = null;
		String line;
		try
		{
			fin = new BufferedReader(reader);

			while ((line = fin.readLine()) != null)	{
				StringTokenizer tokenizer = new StringTokenizer(line, ",");
				Persona p = estraiPersona(tokenizer);
				mappaIscritti.put(p.getId(),p);
			}
			fin.close();
		}
		catch (IOException | NoSuchElementException | NumberFormatException e)
		{
			throw new BadFileFormatException(e);
		}
		return mappaIscritti;
	}
	
	private Persona estraiPersona(StringTokenizer tokenizer) throws BadFileFormatException
	{
		// Roberto, M, 1994-04-24, Toro, capelli neri, occhi castani, 1.78, 61, Bologna, BO,Emilia-Romagna
		String nome = tokenizer.nextToken().trim();
		String sSesso = tokenizer.nextToken().trim();
		
		Sesso sesso = null;
		try {
			sesso = Sesso.valueOfChar(sSesso);
		} catch(IllegalArgumentException e) {
			throw new BadFileFormatException("sesso inesistente: " + sSesso);
		}
		
		String sDataNascita = tokenizer.nextToken().trim();
		LocalDate dataNascita = null; 
		try {
			dataNascita = LocalDate.parse(sDataNascita, DateTimeFormatter.ISO_LOCAL_DATE);
		} catch(DateTimeParseException e) {
			throw new BadFileFormatException("data errata: " + sDataNascita);
		}
					
		String[] chioma = tokenizer.nextToken().trim().split("\\s+");
		if (!chioma[0].equalsIgnoreCase("CAPELLI"))
				throw new BadFileFormatException("manca indicazione CAPELLI");
		Colore capelli = null;
		try {
			capelli = Colore.valueOf(chioma[1].toUpperCase());
		} catch(IllegalArgumentException e) {
			throw new BadFileFormatException("colore capelli errato: " + chioma[1]);
		}
		
		String[] sguardo = tokenizer.nextToken().trim().split("\\s+");
		if (!sguardo[0].equalsIgnoreCase("OCCHI"))
				throw new BadFileFormatException("manca indicazione OCCHI");
		Colore occhi = null;
		try {
			occhi = Colore.valueOf(sguardo[1].toUpperCase());
		} catch(IllegalArgumentException e) {
			throw new BadFileFormatException("colore occhi errato: " + sguardo[1]);
		}

		String numAltezza = tokenizer.nextToken().trim();
		float altezza = Float.parseFloat(numAltezza+"F");

		String numPeso = tokenizer.nextToken().trim();
		int peso = Integer.parseInt(numPeso);

		String citta = tokenizer.nextToken().trim();
		String provincia = tokenizer.nextToken().trim();
		String regione = tokenizer.nextToken().trim();
		
		return new Persona(nome, sesso, dataNascita, calcolaSegnoZodiacale(dataNascita), capelli, occhi, altezza, peso, citta, provincia, regione);
	}

	private SegnoZodiacale calcolaSegnoZodiacale(LocalDate date) {
		for (SegnoZodiacale z : SegnoZodiacale.values())
			if (z.contains(date)) return z;
		throw new IllegalArgumentException("data non coperta nello zodiaco");
	}
	
}
